@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * $Id: gnome-canvas-text.c,v 1.82 2002/04/18 07:44:45 federico Exp $
+ * $Id: gnome-canvas-text.c,v 1.83 2005/02/27 01:52:12 srittau Exp $
  * Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation
  * All rights reserved.
  *
@@ -804,8 +804,7 @@ gnome_canvas_text_set_property (GObject            *object,
 
 	switch (param_id) {
 	case PROP_TEXT:
-		if (text->text)
-			g_free (text->text);
+		g_free (text->text);
 
 		text->text = g_value_dup_string (value);
 		pango_layout_set_text (text->layout, text->text, -1);
@@ -969,6 +968,7 @@ gnome_canvas_text_set_property (GObject            *object,
 			pango_attr_list_unref (text->attr_list);
 
 		text->attr_list = g_value_peek_pointer (value);
+		pango_attr_list_ref (text->attr_list);
 		
 		gnome_canvas_text_apply_attributes (text);
 		text->priv->render_dirty = 1;
@@ -1369,11 +1369,6 @@ gnome_canvas_text_set_markup (GnomeCanvasText *textitem,
 	gchar         *text = NULL;
 	GError        *error = NULL;
 
-	if (textitem->text)
-		g_free (textitem->text);
-	if (textitem->attr_list)
-		pango_attr_list_unref (textitem->attr_list);
-
 	if (markup && !pango_parse_markup (markup, -1,
 					   0,
 					   &attr_list, &text, NULL,
@@ -1385,6 +1380,10 @@ gnome_canvas_text_set_markup (GnomeCanvasText *textitem,
 		return;
 	}
 
+	g_free (textitem->text);
+	if (textitem->attr_list)
+		pango_attr_list_unref (textitem->attr_list);
+
 	textitem->text = text;
 	textitem->attr_list = attr_list;
 
